@@ -3,7 +3,7 @@
    :synopsis: Base classes and functions.
 """
 
-import itertools as itt
+from six.moves import map
 
 
 class Nut(object):
@@ -12,12 +12,13 @@ class Nut(object):
     chained using the '>>' operator. The aim is code with an explicit
     data flow. See the following example using Python iterators versus Nuts:
 
-    >>> from itertools import islice, ifilter
-    >>> print list(islice(ifilter(lambda x: x > 5, xrange(10)), 3))
+    >>> from six.moves import filter, range
+    >>> from itertools import islice
+    >>> list(islice(filter(lambda x: x > 5, range(10)), 3))
     [6, 7, 8]
 
     >>> from nutsflow import Range, Filter, Take, Collect, _
-    >>> print Range(10) >> Filter(_ > 5) >> Take(3) >> Collect()
+    >>> Range(10) >> Filter(_ > 5) >> Take(3) >> Collect()
     [6, 7, 8]
     """
 
@@ -91,7 +92,7 @@ class NutFunction(Nut):
         :return: transformed iterable.
         :rtype: iterable
         """
-        return itt.imap(self, iterable)
+        return map(self, iterable)
 
 
 class NutSource(Nut):
