@@ -3,12 +3,13 @@
    :synopsis: Nuts that perform functions on single stream elements.
 """
 from __future__ import print_function
+from __future__ import absolute_import
 
 import time
 import threading
 
-from common import console
-from factory import nut_function, NutFunction
+from .common import console, is_iterable
+from .factory import nut_function, NutFunction
 
 
 @nut_function
@@ -89,15 +90,19 @@ def Get(x, start, end=None, step=None):
     >>> [(1, 2, 3), (4, 5, 6)] >> Get(0, 3, 2) >> Collect()
     [(1, 3), (4, 6)]
 
+    >>> [(1, 2, 3), (4, 5, 6)] >> Get(None) >> Collect()
+    [(1, 2, 3), (4, 5, 6)]
+
     :param iterable iterable: Any iterable
     :param indexable x: Any indexable input
     :param int start: Start index for columns to extract from x
+           If start = None, x is returned
     :param int end: End index (not inclusive)
     :param int step: Step index (same as slicing)
     :return: Extracted elements
     :rtype: object|list
     """
-    return x[slice(start, end, step) if end else start]
+    return x if start is None else x[slice(start, end, step) if end else start]
 
 
 @nut_function
@@ -116,6 +121,9 @@ def GetCols(x, *columns):
     >>> [[1, 2, 3], [4, 5, 6]] >> GetCols(2, 0) >> Collect()
     [(3, 1), (6, 4)]
 
+    >>> [[1, 2, 3], [4, 5, 6]] >> GetCols((2, 0)) >> Collect()
+    [(3, 1), (6, 4)]
+
     >>> [(1, 2, 3), (4, 5, 6)] >> GetCols(2, 1, 0) >> Collect()
     [(3, 2, 1), (6, 5, 4)]
 
@@ -124,10 +132,13 @@ def GetCols(x, *columns):
 
     :param iterable iterable: Any iterable
     :param indexable container x: Any indexable input
-    :param int|tuple columns: Indicies of elements/columns in x to extract
+    :param int|tuple|args columns: Indicies of elements/columns in x to extract
+       or a tuple with these indices.
     :return: Extracted elements
     :rtype: tuple
     """
+    if len(columns) == 1 and isinstance(columns[0], tuple):
+        columns = columns[0]
     return tuple(x[i] for i in columns)
 
 
@@ -145,7 +156,7 @@ class Counter(NutFunction):
 
         >>> from nutsflow import Consume
         >>> counter = Counter('smallerthan3', lambda x: x < 3, 1)
-        >>> xrange(10) >> counter >> Consume()
+        >>> range(10) >> counter >> Consume()
         >>> counter
         smallerthan3 = 4
 
@@ -240,7 +251,7 @@ def Format(x, fmt):
     :return: Returns inputs as strings formatted as specified
     :rtype: str
     """
-    return fmt.format(*(x if hasattr(x, '__iter__') else [x]))
+    return fmt.format(*(x if is_iterable(x) else [x]))
 
 
 class Print(NutFunction):
@@ -249,7 +260,7 @@ class Print(NutFunction):
     """
 
     def __init__(self, fmtfunc=None, every_sec=0, every_n=0,
-                 filterfunc=lambda x: True):
+                 filterfunc=lambda x: True, end='\n'):
         """
         iterable >> Print(fmtfunc=None, every_sec=0, every_n=0,
                           filterfunc=lambda x: True)
@@ -261,7 +272,7 @@ class Print(NutFunction):
         1
         2
 
-        >>> xrange(10) >> Print(every_n=3) >> Consume()
+        >>> range(10) >> Print(every_n=3) >> Consume()
         2
         5
         8
@@ -271,6 +282,10 @@ class Print(NutFunction):
         2
         4
 
+        >>> [{'val': 1}, {'val': 2}] >> Print('number={val}') >> Consume()
+        number=1
+        number=2
+
         >>> [[1, 2], [3, 4]] >> Print('number={1}:{0}') >> Consume()
         number=2:1
         number=4:3
@@ -280,6 +295,10 @@ class Print(NutFunction):
         char=A
         char=B
 
+        >>> range(5) >> Print('.', end=' ') >> Consume()
+        . . . . .
+
+
         :param object x: Any input
 
         :param string|function fmtfunc: Format string or function.
@@ -289,6 +308,7 @@ class Print(NutFunction):
         :param float every_sec: Print every given second, e.g. to print
                 every 2.5 sec every_sec = 2.5
         :param int every_n: Print every n-th call.
+        :param str end: Ending of text printed.
         :param function filterfunc: Boolean function to filter print.
         :return: Returns input unaltered
         :rtype: object
@@ -298,6 +318,7 @@ class Print(NutFunction):
         self.every_sec = every_sec
         self.every_n = every_n
         self.filterfunc = filterfunc
+        self.end = end
         self.cnt = 0
         self.time = time.time()
 
@@ -324,12 +345,16 @@ class Print(NutFunction):
         if hasattr(x, 'ndim'):  # is it a numpy array?
             x = x.tolist() if x.ndim else x.item()
         if not fmtfunc:
-            console(x)
+            text = x
         elif isinstance(fmtfunc, str):
-            console(fmtfunc.format(*(x if hasattr(x, '__iter__') else [x])))
+            if isinstance(x, dict):
+                text = fmtfunc.format(**x)
+            else:
+                text = fmtfunc.format(*(x if is_iterable(x) else [x]))
         elif hasattr(fmtfunc, '__call__'):
-            console(fmtfunc(x))
+            text = fmtfunc(x)
         else:
             raise ValueError('Invalid format ' + str(fmtfunc))
+        console(text, end=self.end)
 
         return x
