@@ -2,26 +2,30 @@
 .. module:: processor
    :synopsis: Nuts that process iterables and return iterables.
 """
+from __future__ import print_function, division
 
 import tempfile
 import shutil
 import os
 import time
+import six
+import sys
 
-import cPickle as pickle
 import os.path as osp
 import itertools as itt
-import iterfunction as itf
 import random as rnd
 import multiprocessing as mp
 import collections as cl
 
-from base import Nut
-from common import as_tuple, as_set, console
-from factory import nut_processor
-from function import Identity
-from sink import Consume, Collect
-from nutsflow.common import timestr
+from inspect import isfunction, ismethod, isbuiltin
+from six.moves import cPickle as pickle
+from six.moves import map, filter, filterfalse, zip, range
+from . import iterfunction as itf
+from .base import Nut, NutFunction
+from .common import as_tuple, as_list, as_set, console, timestr, is_iterable
+from .factory import nut_processor
+from .function import Identity
+from .sink import Consume, Collect, Head
 
 
 @nut_processor
@@ -73,6 +77,35 @@ def Slice(iterable, start=None, *args, **kwargs):
     return itt.islice(iterable, start, *args, **kwargs)
 
 
+@nut_processor
+def Window(iterable, n=2):
+    """
+    iterable >> Window(n)
+
+    Sliding window of size n over elements in iterable.
+
+    >>> [1, 2, 3, 4] >> Window() >> Collect()
+    [(1, 2), (2, 3), (3, 4)]
+
+    >>> [1, 2, 3, 4] >> Window(3) >> Collect()
+    [(1, 2, 3), (2, 3, 4)]
+
+    >>> 'test' >> Window(2) >> Map(''.join) >> Collect()
+    ['te', 'es', 'st']
+
+    :param iterable iterable: Any iterable
+    :param int n: Size of window
+    :return: iterator with tuples of length n
+    :rtype: iterator over tuples
+    """
+    it = iter(iterable)
+    win = cl.deque(it >> Take(n), n)
+    yield tuple(win)
+    for e in it:
+        win.append(e)
+        yield tuple(win)
+
+
 @nut_processor
 def Concat(iterable, *iterables):
     """
@@ -143,7 +176,7 @@ def Zip(iterable, iterable2=None, *iterables):
     :return: Zipped elements from iterables.
     :rtype: iterator over tuples
     """
-    return itt.izip(iterable, iterable2, *iterables)
+    return zip(iterable, iterable2, *iterables)
 
 
 @nut_processor
@@ -164,7 +197,74 @@ def ZipWith(iterable, f, *iterables):
     :rtype: iterator
     """
     iterables = [iterable] + list(iterables)
-    return itt.starmap(f, itt.izip(*iterables))
+    return itt.starmap(f, zip(*iterables))
+
+
+@nut_processor
+def Append(iterable, items):
+    """
+    iterable >> Append(items)
+
+    Append item(s) to lists/tuples in iterable.
+
+    >>> [(1, 2), (3, 4)] >> Append('X') >> Collect()
+    [(1, 2, 'X'), (3, 4, 'X')]
+
+    >>> items = ['a', 'b']
+    >>> [(1, 2), (3, 4)] >> Append(items) >> Collect()
+    [(1, 2, 'a'), (3, 4, 'b')]
+
+    >>> items = [('a', 'b'), ('c', 'd')]
+    >>> [(1, 2), (3, 4)] >> Append(items) >> Collect()
+    [(1, 2, 'a', 'b'), (3, 4, 'c', 'd')]
+
+    >>> from nutsflow import Enumerate
+    >>> [(1, 2), (3, 4)] >> Append(Enumerate()) >> Collect()
+    [(1, 2, 0), (3, 4, 1)]
+
+    :param iterable iterable iterable: Any iterable over tuples or lists
+    :param iterable|object items: A single object or an iterable over objects.
+    :return: iterator where items are appended to the iterable elements.
+    :rtype: iterator over tuples
+    """
+    items = items if is_iterable(items) else itt.repeat(items)
+    for elem, item in zip(iterable, items):
+        yield tuple(elem) + as_tuple(item)
+
+
+@nut_processor
+def Insert(iterable, index, items):
+    """
+    iterable >> Insert(index, items)
+
+    Insert item(s) into lists/tuples in iterable.
+
+    >>> [(1, 2), (3, 4)] >> Insert(1, 'X') >> Collect()
+    [(1, 'X', 2), (3, 'X', 4)]
+
+    >>> items = ['a', 'b']
+    >>> [(1, 2), (3, 4)] >> Insert(2, items) >> Collect()
+    [(1, 2, 'a'), (3, 4, 'b')]
+
+    >>> items = [('a', 'b'), ('c', 'd')]
+    >>> [(1, 2), (3, 4)] >> Insert(1, items) >> Collect()
+    [(1, 'a', 'b', 2), (3, 'c', 'd', 4)]
+
+    >>> from nutsflow import Enumerate
+    >>> [(1, 2), (3, 4)] >> Insert(0, Enumerate()) >> Collect()
+    [(0, 1, 2), (1, 3, 4)]
+
+    :param iterable iterable iterable: Any iterable over tuples or lists
+    :param int index: Index at which position items are inserted.
+    :param iterable|object items: A single object or an iterable over objects.
+    :return: iterator where items are inserted into the iterable elements.
+    :rtype: iterator over tuples
+    """
+    items = items if is_iterable(items) else itt.repeat(items)
+    for elem, item in zip(iterable, items):
+        elem = list(elem)
+        head, tail = elem[:index], elem[index:]
+        yield tuple(head + as_list(item) + tail)
 
 
 Dedupe = nut_processor(itf.unique)
@@ -184,50 +284,82 @@ if iterable is long and many elements are unique!
 >>> data >> Dedupe(_[1]) >> Collect()
 [(1, 'a'), (3, 'b')]
 
-:param iterable iterable: Any iterable, e.g. list, xrange, ...
+:param iterable iterable: Any iterable, e.g. list, range, ...
 :param key: Function used to compare for equality.
 :return: Iterator over unique elements.
 :rtype: Iterator
 """
 
-Chunk = nut_processor(itf.chunked)
-"""
-iterable >> Chunk(n)
 
-Split iterable in chunks of size n, where each chunk is also an iterator.
-see also GroupBySorted(), ChunkWhen(), ChunkBy()
+@nut_processor
+def Chunk(iterable, n, container=None):
+    """
+    iterable >> Chunk(n, container=None)
 
->>> for chunk in Range(5) >> Chunk(2):
->>> ... print list(chunk)
-[0, 1]
-[2, 3]
-[4]
+    Split iterable in chunks of size n, where each chunk is also an iterator
+    if no container is provided.
+    see also GroupBySorted(), ChunkWhen(), ChunkBy()
 
-:param iterable iterable: Any iterable, e.g. list, xrange, ...
-:param int n: Chunk size
-:return: Chunked iterable
-:rtype: Iterator over iterators
-"""
+    >>> from nutsflow import Range, Map, Print, Join, Consume, Collect
+    >>> Range(5) >> Chunk(2) >> Map(list) >> Print() >> Consume()
+    [0, 1]
+    [2, 3]
+    [4]
+
+    The code can be shortend by providing a container in Chunk():
+
+    >>> Range(5) >> Chunk(2, list) >> Print() >> Consume()
+    [0, 1]
+    [2, 3]
+    [4]
+
+    >>> Range(6) >> Chunk(3, Join('_')) >> Print() >> Consume()
+    0_1_2
+    3_4_5
+
+    >>> Range(6) >> Chunk(3, sum) >> Collect()
+    [3, 12]
+
+    :param iterable iterable: Any iterable, e.g. list, range, ...
+    :param int n: Chunk size
+    :param container container: Some container, e.g. list, set, dict
+           that can be filled from an iterable
+    :return: Chunked iterable
+    :rtype: Iterator over iterators or containers
+    """
+    chunks = itf.chunked(iterable, n)
+    return map(container, chunks) if container else chunks
 
 
 class ChunkWhen(Nut):
-    def __init__(self, func):
+    def __init__(self, func, container=None):
         """
-        iterable >> ChunkWhen(func)
+        iterable >> ChunkWhen(func, container=None)
 
         Chunk iterable and create new chunk every time func returns True.
         see also GroupBySorted(), Chunk(), ChunkBy()
 
         >>> from nutsflow import Map, Join, Collect
+        >>> func = lambda x: x == 1
+        >>> [1,2,1,3,1,4,5] >> ChunkWhen(func, tuple) >> Collect()
+        [(1, 2), (1, 3), (1, 4, 5)]
+
+        >>> func = lambda x: x == 1
+        >>> [1,2,1,3,1,4,5] >> ChunkWhen(func, sum) >> Collect()
+        [3, 4, 10]
+
         >>> func = lambda x: x == '|'
-        >>> '0|12|345|6' >> ChunkWhen(func) >> Map(Join()) >> Collect()
+        >>> '0|12|345|6' >> ChunkWhen(func, Join()) >> Collect()
         ['0', '|12', '|345', '|6']
 
         :param function func: Boolean function that indicates chunks.
             New chunk is created if return value is True.
+        :param container container: Some container, e.g. list, set, dict
+           that can be filled from an iterable
         """
         self.cnt = 0
         self.func = func
+        self.container = container
 
     def _key(self, x):
         """ Return keys (= counter) for groups (=chunks)"""
@@ -238,33 +370,37 @@ class ChunkWhen(Nut):
     def __rrshift__(self, iterable):
         """
         :param any iterable iterable: iterable to create chunks for.
-        :return: Iterator over chunks, where each chunk is an iterator itself.
-        :rtype: iterator over iterators
+        :return: Iterator over chunks, where each chunk is an iterator itself
+                 if no container is provided
+        :rtype: iterator over iterators or containers
         """
-        return iterable >> ChunkBy(self._key)
+        return iterable >> ChunkBy(self._key, self.container)
 
 
 @nut_processor
-def ChunkBy(iterable, func):
+def ChunkBy(iterable, func, container=None):
     """
-    iterable >> ChunkBy(func)
+    iterable >> ChunkBy(func, container=None)
 
     Chunk iterable and create chunk every time func changes its return value.
     see also GroupBySorted(), Chunk(), ChunkWhen()
 
-    >>> [1,1, 2, 3,3,3] >> ChunkBy(lambda x: x) >> Map(list) >> Collect()
-    [[1, 1], [2], [3, 3, 3]]
+    >>> [1,1, 2, 3,3,3] >> ChunkBy(lambda x: x, tuple) >> Collect()
+    [(1, 1), (2,), (3, 3, 3)]
 
-    >>> [1,1, 2, 3,3,3] >> ChunkBy(lambda x: x < 3) >> Map(list) >> Collect()
-    [[1, 1, 2], [3, 3, 3]]
+    >>> [1,1, 2, 3,3,3] >> ChunkBy(lambda x: x < 3, tuple)  >> Collect()
+    [(1, 1, 2), (3, 3, 3)]
 
-    :param iterable iterable: Any iterable, e.g. list, xrange, ...
-    :param function func: Functions the iterable is chunked by 
+    :param iterable iterable: Any iterable, e.g. list, range, ...
+    :param function func: Functions the iterable is chunked by
+    :param container container: Some container, e.g. list, set, dict
+           that can be filled from an iterable
     :return: Chunked iterable
-    :rtype: Iterator over iterators
+    :rtype: Iterator over iterators or containers
     """
     groupiter = itt.groupby(iterable, func)
-    return itt.imap(lambda t: t[1], groupiter)
+    chunks = map(lambda t: t[1], groupiter)
+    return map(container, chunks) if container else chunks
 
 
 Cycle = nut_processor(itt.cycle)
@@ -277,7 +413,7 @@ large!
 >>> [1, 2] >> Cycle() >> Take(5) >> Collect()
 [1, 2, 1, 2, 1]
 
-:param iterable iterable: Any iterable, e.g. list, xrange, ...
+:param iterable iterable: Any iterable, e.g. list, range, ...
 :return: Cycled input iterable
 :rtype: Iterator
 """
@@ -312,32 +448,50 @@ def Flatten(iterable):
 
 
 @nut_processor
-def FlattenCol(iterable, columns):
+def FlattenCol(iterable, cols):
     """
-    iterable >> FlattenCol(columns)
+    iterable >> FlattenCol(cols)
 
     Flattens the specified columns of the tuples/iterables within the iterable.
     Only one level is flattened.
 
+    (1 3)  (5 7)
+    (2 4)  (6 8)   >> FlattenCol((0,1) >>   (1 3)  (2 4)  (5 7)  (6 8)
+
+    If a column contains a single element (instead of an iterable) it is 
+    wrapped into a repeater. This allows to flatten columns that are iterable
+    together with non-iterable columns, e.g.
+
+    (1 3)  (6 7)
+    (2  )  (  8)   >> FlattenCols((0,1) >>   (1 3)  (2 3)  (6 7)  (6 8)
+
     >>> from nutsflow import Collect
     >>> data = [([1, 2], [3, 4]), ([5, 6], [7, 8])]
     >>> data >> FlattenCol(0) >> Collect()
     [(1,), (2,), (5,), (6,)]
 
+    >>> data >> FlattenCol((0, 1)) >> Collect()
+    [(1, 3), (2, 4), (5, 7), (6, 8)]
+
     >>> data >> FlattenCol((1, 0)) >> Collect()
     [(3, 1), (4, 2), (7, 5), (8, 6)]
 
     >>> data >> FlattenCol((1, 1, 0)) >> Collect()
     [(3, 3, 1), (4, 4, 2), (7, 7, 5), (8, 8, 6)]
 
+    >>> data = [([1, 2], 3), (6, [7, 8])]
+    >>> data >> FlattenCol((0, 1)) >> Collect()
+    [(1, 3), (2, 3), (6, 7), (6, 8)]
+
     :param iterable iterable: Any iterable.
     :params int|tuple columns: Column index or indices
     :return: Flattened columns of iterable
     :rtype: generator
     """
-    columns = as_tuple(columns)
+    cols = as_tuple(cols)
+    get = lambda e: e if is_iterable(e) else itt.repeat(e)
     for es in iterable:
-        for e in itt.izip(*[es[c] for c in columns]):
+        for e in zip(*[get(es[c]) for c in cols]):
             yield e
 
 
@@ -360,7 +514,7 @@ iterable >> Map(func) >> Flatten()
 :rtype: Iterator
 """
 
-Map = nut_processor(itt.imap, 1)
+Map = nut_processor(map, 1)
 """
 iterable >> Map(func, *iterables)
 
@@ -383,7 +537,7 @@ See https://docs.python.org/2/library/itertools.html#itertools.imap
 :rtype: Iterator
 """
 
-Filter = nut_processor(itt.ifilter, None)
+Filter = nut_processor(filter, None)
 """
 iterable >> Filter(func)
 
@@ -399,7 +553,7 @@ See https://docs.python.org/2/library/itertools.html#itertools.ifilter
 :rtype: Iterator
 """
 
-FilterFalse = nut_processor(itt.ifilterfalse, None)
+FilterFalse = nut_processor(filterfalse, None)
 """
 iterable >> FilterFalse(func)
 
@@ -417,6 +571,35 @@ See https://docs.python.org/2/library/itertools.html#itertools.ifilterfalse
 :rtype: Iterator
 """
 
+@nut_processor
+def FilterCol(iterable, columns, func):
+    """
+    iterable >> FilterCol(columns, func)
+
+    Filter elements from iterable based on predicate function and
+    specified column(s).
+
+    >>> is_even = lambda n: n % 2 == 0
+    >>> [(0, 'e'), (1, 'o'), (2, 'e')] >> FilterCol(0, is_even) >> Collect()
+    [(0, 'e'), (2, 'e')]
+
+
+    :param iterable iterable: Any iterable
+    :param int|tuple columns: Column or columns to extract from each
+        element before passing it on to the predicate function.
+    :param function func: Predicate function. Element is removed if False.
+    :return: Filtered iterable
+    :rtype: Iterator
+    """
+    cols = as_tuple(columns)
+    if len(cols) == 1:
+        extract = lambda es: es[columns]
+    else:
+        extract = lambda es: [es[i] for i, e in enumerate(es) if i in cols]
+    for es in iterable:
+        if func(extract(es)):
+            yield es
+
 Partition = nut_processor(itf.partition)
 """
 partition1, partition2 = iterable >> Partition(func)
@@ -429,7 +612,7 @@ Split iterable into two partitions based on predicate function
 >>> larger >> Collect()
 [3, 4]
 
-:param iterable: Any iterable, e.g. list, xrange, ...
+:param iterable: Any iterable, e.g. list, range, ...
 :param pred: Predicate function.
 :return: Partition iterators
 :rtype: Two iterators
@@ -579,7 +762,7 @@ def Drop(iterable, n):
 
 
 @nut_processor
-def Pick(iterable, p_n, rand=rnd.Random()):
+def Pick(iterable, p_n, rand=None):
     """
     iterable >> Pick(p_n)
 
@@ -587,7 +770,8 @@ def Pick(iterable, p_n, rand=rnd.Random()):
     otherwise pick randomly with probability p_n.
 
     >>> from nutsflow import Range, Collect
-    
+    >>> from nutsflow.common import StableRandom
+
     >>> [1, 2, 3, 4] >> Pick(0.0) >> Collect()
     []
 
@@ -595,8 +779,8 @@ def Pick(iterable, p_n, rand=rnd.Random()):
     [1, 2, 3, 4]
 
     >>> import random as rnd
-    >>> Range(10) >> Pick(0.5, rnd.Random(0)) >> Collect()
-    [2, 3, 5, 7, 8]
+    >>> Range(10) >> Pick(0.5, StableRandom(1)) >> Collect()
+    [0, 4, 5, 6, 8, 9]
 
     >>> [1, 2, 3, 4] >> Pick(2) >> Collect()
     [1, 3]
@@ -604,10 +788,12 @@ def Pick(iterable, p_n, rand=rnd.Random()):
     :param iterable iterable: Any iterable
     :param float|int p_n: Probability p in [0, 1] or
         integer n for every n-th element
-    :param Random rand: Random number generator to be used.
+    :param Random|None rand: Random number generator. If None, random.Random()
+        is used.
     :return: Iterator over picked elements.
     :rtype: iterator
     """
+    rand = rnd.Random() if rand is None else rand
     if isinstance(p_n, int):
         if p_n < 0:
             raise ValueError('p_n must not be negative ' + str(p_n))
@@ -629,22 +815,22 @@ def GroupBy(iterable, keycol=lambda x: x, nokey=False):
     If the iterable is sorted use GroupBySorted() instead.
     see also Chunk(), ChunkWhen(), ChunkBy()
 
-    >>> from nutsflow import Collect
+    >>> from nutsflow import Sort
 
-    >>> [1, 2, 1, 1, 3] >> GroupBy() >> Collect()
+    >>> [1, 2, 1, 1, 3] >> GroupBy() >> Sort()
     [(1, [1, 1, 1]), (2, [2]), (3, [3])]
 
-    >>> [1, 2, 1, 1, 3] >> GroupBy(nokey=True) >> Collect()
+    >>> [1, 2, 1, 1, 3] >> GroupBy(nokey=True) >> Sort()
     [[1, 1, 1], [2], [3]]
 
-    >>> ['--', '+++', '**'] >> GroupBy(len) >> Collect()
+    >>> ['--', '+++', '**'] >> GroupBy(len) >> Sort()
     [(2, ['--', '**']), (3, ['+++'])]
 
-    >>> ['a3', 'b2', 'c1'] >> GroupBy(1) >> Collect()
-    [('1', ['c1']), ('3', ['a3']), ('2', ['b2'])]
+    >>> ['a3', 'b2', 'c1'] >> GroupBy(1) >> Sort()
+     [('1', ['c1']), ('2', ['b2']), ('3', ['a3'])]
 
-    >>> [(1,3), (2,2), (3,1)] >> GroupBy(1, nokey=True) >> Collect()
-    [[(3, 1)], [(2, 2)], [(1, 3)]]
+    >>> [(1,3), (2,2), (3,1)] >> GroupBy(1, nokey=True) >> Sort()
+    [[(1, 3)], [(2, 2)], [(3, 1)]]
 
     :param iterable iterable: Any iterable
     :param int|function keycol: Column index or key function.
@@ -658,7 +844,7 @@ def GroupBy(iterable, keycol=lambda x: x, nokey=False):
     for e in iterable:
         key = keycol(e) if isfunc else e[keycol]
         groups[key].append(e)
-    return groups.itervalues() if nokey else groups.iteritems()
+    return six.itervalues(groups) if nokey else six.iteritems(groups)
 
 
 @nut_processor
@@ -678,7 +864,7 @@ def GroupBySorted(iterable, keycol=lambda x: x, nokey=False):
     
     >>> @nut_sink
     ... def ViewResult(iterable):
-    ...     return iterable >> Map(lambda (k, es): (k, list(es))) >> Collect()
+    ...     return iterable >> Map(lambda t: (t[0], list(t[1]))) >> Collect()
 
     >>> [1, 1, 1, 2, 3] >> GroupBySorted() >> ViewResult()
     [(1, [1, 1, 1]), (2, [2]), (3, [3])]
@@ -699,7 +885,7 @@ def GroupBySorted(iterable, keycol=lambda x: x, nokey=False):
     isfunc = hasattr(keycol, '__call__')
     key = keycol if isfunc else lambda x: x[keycol]
     groupiter = itt.groupby(iterable, key)
-    return itt.imap(lambda (k, v): v, groupiter) if nokey else groupiter
+    return map(lambda k_v: k_v[1], groupiter) if nokey else groupiter
 
 
 @nut_processor
@@ -722,12 +908,12 @@ def Clone(iterable, n):
     :rtype: generator
     """
     for e in iterable:
-        for _ in xrange(n):
+        for _ in range(n):
             yield e
 
 
 @nut_processor
-def Shuffle(iterable, buffersize, rand=rnd.Random()):
+def Shuffle(iterable, buffersize, rand=None):
     """
     iterable >> Shuffle(buffersize)
 
@@ -738,23 +924,28 @@ def Shuffle(iterable, buffersize, rand=rnd.Random()):
     'window' of the shuffle is limited to buffersize.
     Note that for buffersize = 1 no shuffling occurs.
 
-    In the following example rand = rnd.Random(0) is used to create a fixed
-    shuffle. Usually, this is not what you want. Use the default
-    rand=rnd.Random() instead.
+    In the following example rand = StableRandom(0) is used to create a fixed
+    sequence that stable across Python version 2.x and 3.x. Usually, this is
+    not what you want. Use the default rand=None which uses random.Random()
+    instead.
 
     >>> from nutsflow import Range, Collect
-    >>> Range(10) >> Shuffle(5, rnd.Random(0)) >> Collect()
-    [1, 5, 3, 0, 6, 2, 8, 9, 7, 4]
+    >>> from nutsflow.common import StableRandom
 
-    >>> Range(10) >> Shuffle(1, rnd.Random(0)) >> Collect()
+    >>> Range(10) >> Shuffle(5, StableRandom(0)) >> Collect()
+    [4, 2, 3, 6, 7, 0, 1, 9, 5, 8]
+
+    >>> Range(10) >> Shuffle(1, StableRandom(0)) >> Collect()
     [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
 
     :param iterable iterable: Any iterable
     :param int buffersize: Number of elements stored in shuffle buffer.
-    :param random.Random rand: Random number generator.
+    :param Random|None rand: Random number generator. If None,
+           random.Random() is used.
     :return: Generator over shuffled elements
     :rtype: generator
     """
+    rand = rnd.Random() if rand is None else rand
     iterable = iter(iterable)
     buffer = list(itf.take(iterable, buffersize))
     rand.shuffle(buffer)
@@ -822,7 +1013,7 @@ def MapMulti(iterable, *funcs):
     :rtype: (iterator, ...)
     """
     tees = itt.tee(iterable, len(funcs))
-    return [itt.imap(f, t) for f, t in zip(funcs, tees)]
+    return [map(f, t) for f, t in zip(funcs, tees)]
 
 
 # Don't use @nut_processor here. Creating Pool is expensive!
@@ -868,7 +1059,7 @@ class Cache(Nut):
     to file system and loads them the next time instead of recomputing.
     """
 
-    def __init__(self, storage='disk'):
+    def __init__(self, cachepath=None, clearcache=True):
         """
         iterable >> Cache()
 
@@ -879,33 +1070,45 @@ class Cache(Nut):
         .. code:: python
 
             with Cache() as cache:
-                data = xrange(100)
-                for i in xrange(10):
+                data = range(100)
+                for i in range(10):
                     data >> expensive_op >> cache >> process(i) >> Consume()
 
 
         .. code:: python
 
             cache = Cache()
-            for _ in xrange(100)
+            for _ in range(100)
                 data >> expensive_op >> cache >> Collect()
             cache.clear()
 
+
+        .. code:: python
+
+            with Cache('path/to/mycache') as cache:
+                for _ in range(100)
+                    data >> expensive_op >> cache >> Collect()
+
         :param iterable iterable: Any iterable
-        :param string storage: Currently only 'disk' mode.
+        :param string cachepath: Path to a folder that stores the cached
+           objects. If the path does not exist it will be created. The path
+           with all its contents will be deleted when the cache is deleted.
+           For cachepath=None a temporary folder will be created. Path to
+           this folder is available in cache.path.
+        :param bool clearcache: Clear left-over cache if it exists.
         :return: Iterator over elements
         :rtype: iterator
         """
-
-        self._dirpath = None
-        self.storage = storage  # currently not used. Could be 'disk', 'memory'
-        if storage != 'disk':
-            raise ValueError('Unsupported storage: ' + storage)
+        self.path = None
+        self._cachepath = cachepath
+        self._clearcache = clearcache
+        if clearcache and cachepath and os.path.exists(cachepath):
+            shutil.rmtree(cachepath)  # delete it.
 
     def clear(self):
         """Clear cache"""
-        shutil.rmtree(self._dirpath, ignore_errors=True)
-        self._dirpath = None
+        shutil.rmtree(self.path, ignore_errors=True)
+        self.path = None
 
     def _fpath(self, idx):
         """
@@ -915,8 +1118,7 @@ class Cache(Nut):
         :rtype: str
         """
         fname = 'cache_{0:010d}.pkl'.format(idx)
-        fpath = osp.join(self._dirpath, fname)
-        return fpath
+        return osp.join(self.path, fname)
 
     def _cache_fpaths(self):
         """
@@ -925,8 +1127,17 @@ class Cache(Nut):
         :return: Filepaths to pickle files.
         :rtype: list of strings
         """
-        dirpath = self._dirpath
-        return sorted(osp.join(dirpath, name) for name in os.listdir(dirpath))
+        path = self._cachepath if self._cachepath else self.path
+        return sorted(osp.join(path, n) for n in os.listdir(path))
+
+    def _create_cache(self):
+        """
+        Create either user-defined cache path or temporary path for cache data.
+        """
+        if self._cachepath:  # user defined cache path.
+            if not os.path.exists(self._cachepath):
+                os.makedirs(self._cachepath)  # create cache
+        self.path = self._cachepath if self._cachepath else tempfile.mkdtemp()
 
     def __enter__(self):
         """
@@ -943,20 +1154,30 @@ class Cache(Nut):
         """
         self.clear()
 
+    def __iter__(self):
+        """
+        Return iterator over cached elements.
+
+        :return: Generator over cached elements
+        :rtype: Generator
+        """
+        for fpath in self._cache_fpaths():
+            with open(fpath, 'rb') as f:
+                yield pickle.load(f)
+
     def __rrshift__(self, iterable):
         """
         Return elements in iterable.
 
         :param iterable iterable: Any iterable
-        :return: Iterable over same elements as input iterable.
-        :rtype: iterable
+        :return: Generator over same elements as input iterable.
+        :rtype: Generator
         """
-        if self._dirpath:
-            for fpath in self._cache_fpaths():
-                with open(fpath, 'rb') as f:
-                    yield pickle.load(f)
+        if self.path or (self._cachepath and not self._clearcache):
+            for e in self.__iter__():
+                yield e
         else:
-            self._dirpath = tempfile.mkdtemp()
+            self._create_cache()
             for i, e in enumerate(iterable):
                 with open(self._fpath(i), 'wb') as f:
                     pickle.dump(e, f, pickle.HIGHEST_PROTOCOL)
@@ -986,7 +1207,7 @@ def Prefetch(iterable, num_prefetch=1):
 
 
 class PrintProgress(Nut):
-    def __init__(self, data, every_sec=10.0):
+    def __init__(self, data, title='progress:', every_sec=10.0):
         """
         iterable >> PrintProgress(data, every_sec=10.0)
 
@@ -995,34 +1216,110 @@ class PrintProgress(Nut):
         For long running computations and Estimated time of arrival (eta) is
         printed as well
 
-        xrange(10) >> PrintProgress(10, 0) >> Consume()
+        range(10) >> PrintProgress(10, 'numbers:', 0) >> Consume()
 
         :param iterable iterable: Any iterable
         :param int data: Number of elements in iterable or realized iterable.
                If data is provided it must not be an iterator since it will be
                consumed!
+        :param str title: Title of progress print out (prefix text)
         :param float every_sec: Progress is printed every 'every_sec' seconds.
+
         :return: Iterator over input elements
         :rtype: iterator
         """
         self.n = (data if isinstance(data, int) else len(data)) - 1
+        self.title = title
         self.every_sec = every_sec
 
     def __rrshift__(self, iterable):
         etafmt = '(eta: {:d}:{:02d}:{:02d})'
         endfmt = '(took: {:d}:{:02d}:{:02d})'
-        start_time = time.clock()
-        up_time = time.clock()
+        start_time = time.time()
+        up_time = time.time()
         for i, e in enumerate(iterable):
-            if (time.clock() - up_time) >= self.every_sec:
-                up_time = time.clock()
-                per_done = 100 * i / self.n
-                sec_consumed = int(time.clock() - start_time)
+            if (time.time() - up_time) >= self.every_sec:
+                up_time = time.time()
+                per_done = int(100 * i / self.n)
+                sec_consumed = int(time.time() - start_time)
                 eta = sec_consumed * (self.n / float(i) - 1) if i else 0
                 tstr = timestr(eta, etafmt)
-                text = '\rprogress: {}% {}'.format(per_done, tstr)
+                text = '\r{} {}% {}'.format(self.title, per_done, tstr)
                 console(text, end='')
             yield e
-        duration = int(time.clock() - start_time)
-        text = '\rprogress: 100% {}'.format(timestr(duration, endfmt))
+        duration = int(time.time() - start_time)
+        text = '\r{} 100% {}'.format(self.title, timestr(duration, endfmt))
         console(text)
+
+
+@nut_processor
+def Try(iterable, func, default='STDERR'):
+    """
+    iterable >> Try(nut)
+
+    Exception handling for (nut) functions. If the wrapped nut or function 
+    raises an exception it is caught and handled with the provided handler.
+    Per default the exception and the value causing it are printed.
+    Furthermore a default value can be specified that is returned instead
+    of the nut output if an exception occurs. Per default no output is
+    returned but an error message printed (STDERR).
+
+    NOTE: In the following examples 'STDOUT' is used only to verify the
+    error message within the doctest. In production code use the default
+    value of 'STDERR'.
+
+    >>> from nutsflow import Try, Collect, nut_function  
+
+    >>> [10, 2, 1] >> Try(lambda x : 10//x) >> Collect()
+    [1, 5, 10]
+    >>> [10, 0, 1] >> Try(lambda x : 10//x, 'STDOUT') >> Collect()
+    ERROR: 0 : integer division or modulo by zero
+    [1, 10]
+
+    >>> Div = nut_function(lambda x : 10//x)
+    >>> [10, 2, 1] >> Try(Div()) >> Collect()
+    [1, 5, 10]
+    >>> [10, 0, 1] >> Try(Div(), 'STDOUT') >> Collect()
+    ERROR: 0 : integer division or modulo by zero
+    [1, 10]
+    >>> [10, 0, 1] >> Try(Div(), -1) >> Collect()
+    [1, -1, 10]
+
+    >>> handlezero = lambda x, e: 'FAILED: '+str(x)
+    >>> [10, 0, 1] >> Try(Div(), handlezero) >> Collect()
+    [1, 'FAILED: 0', 10]
+
+    >>> handlezero = lambda x, e: str(e)
+    >>> [10, 0, 1] >> Try(Div(), handlezero) >> Collect()
+    [1, 'integer division or modulo by zero', 10]
+
+    :param iterable iterable: Iterable the nut operates on.
+    :param function|NutFunction func: (Nut) function that is wrapped 
+       for exception handling. Can be a plain Python function/method as well.
+    :param Object default: Return value if exception occurs. 
+       If default = 'IGNORE', no value is returned and no error is printed.
+       If default = 'STDERR', no value is returned, error is printed to stderr.
+       If default = 'STDOUT', no value is returned, error is printed to stdout.
+       If default is function that takes element x and  exception e
+          as parameters its result is returned and no error is printed.
+       Otherwise the default value is returned and no error is printed.
+    :return: Iterator over input elements transformed by provided nut.
+    :rtype: iterator
+    """
+    if not (isinstance(func, NutFunction) or isfunction(func) or
+                isbuiltin(func) or ismethod(func)):
+        raise TypeError('Need (nut) function in Try() :' + str(func))
+    for x in iterable:
+        try:
+            yield func(x)
+        except Exception as e:
+            if default == 'IGNORE':
+                pass
+            elif default == 'STDERR':
+                print('ERROR:', x, ':', e, file=sys.stderr)
+            elif default == 'STDOUT':
+                print('ERROR:', x, ':', e)
+            elif hasattr(default, '__call__'):
+                yield default(x, e)
+            else:
+                yield default
