@@ -2,14 +2,15 @@
 .. module:: test_processor
    :synopsis: Unit tests for processor module
 """
+from __future__ import print_function, division
 
 import pytest
+import os
 
-import random as rnd
-
+from six.moves import range
 from nutsflow import *
 from nutsflow import _
-from nutsflow.common import Redirect
+from nutsflow.common import Redirect, StableRandom
 
 
 def test_Take():
@@ -26,6 +27,16 @@ def test_Slice():
     assert [1, 2, 3, 4] >> Slice(0, 4, 2) >> Collect() == [1, 3]
 
 
+def test_Window():
+    assert [] >> Window() >> Collect() == [()]
+
+    expected = [(1, 2), (2, 3), (3, 4)]
+    assert [1, 2, 3, 4] >> Window() >> Collect() == expected
+
+    expected = [(1, 2, 3), (2, 3, 4)]
+    assert [1, 2, 3, 4] >> Window(3) >> Collect() == expected
+
+
 def test_Concat():
     assert [] >> Concat([]) >> Collect() == []
     assert [1, 2] >> Concat([]) >> Collect() == [1, 2]
@@ -59,13 +70,49 @@ def test_ZipWith():
     assert [1, 2] >> ZipWith(add3, [2, 3], [4, 5]) >> Collect() == [7, 10]
 
 
+def test_Append():
+    assert [] >> Append('X') >> Collect() == []
+
+    expected = [(1, 2, 'X'), (3, 4, 'X')]
+    assert [(1, 2), (3, 4)] >> Append('X') >> Collect() == expected
+
+    items = ['a', 'b']
+    expected = [(1, 2, 'a'), (3, 4, 'b')]
+    assert [(1, 2), (3, 4)] >> Append(items) >> Collect() == expected
+
+    items = [('a', 'b'), ('c', 'd')]
+    expected = [(1, 2, 'a', 'b'), (3, 4, 'c', 'd')]
+    assert [(1, 2), (3, 4)] >> Append(items) >> Collect() == expected
+
+    expected = [(1, 2, 0), (3, 4, 1)]
+    assert [(1, 2), (3, 4)] >> Append(Enumerate()) >> Collect() == expected
+
+
+def test_Insert():
+    assert [] >> Insert(1, 'X') >> Collect() == []
+
+    expected = [(1, 'X', 2), (3, 'X', 4)]
+    assert [(1, 2), (3, 4)] >> Insert(1, 'X') >> Collect() == expected
+
+    items = ['a', 'b']
+    expected = [(1, 2, 'a'), (3, 4, 'b')]
+    assert [(1, 2), (3, 4)] >> Insert(2, items) >> Collect() == expected
+
+    items = [('a', 'b'), ('c', 'd')]
+    expected = [(1, 'a', 'b', 2), (3, 'c', 'd', 4)]
+    assert [(1, 2), (3, 4)] >> Insert(1, items) >> Collect() == expected
+
+    expected = [(0, 1, 2), (1, 3, 4)]
+    assert [(1, 2), (3, 4)] >> Insert(0, Enumerate()) >> Collect() == expected
+
+
 def test_Dedupe():
     assert [] >> Dedupe() >> Collect() == []
     assert [2, 3, 1, 1, 2, 4] >> Dedupe() >> Collect() == [2, 3, 1, 4]
 
     data = [(1, 'a'), (2, 'a'), (3, 'b')]
     expected = [(1, 'a'), (3, 'b')]
-    assert data >> Dedupe(key=lambda (x, y): y) >> Collect() == expected
+    assert data >> Dedupe(key=lambda t: t[1]) >> Collect() == expected
     assert data >> Dedupe(_[1]) >> Collect() == expected
 
 
@@ -78,6 +125,7 @@ def test_Chunk():
     assert [] >> Chunk(2) >> Collect() == []
     expected = [(0, 1), (2, 3), (4, 5), (6,)]
     assert Range(7) >> Chunk(2) >> Map(tuple) >> Collect() == expected
+    assert Range(7) >> Chunk(2, tuple) >> Collect() == expected
 
 
 def test_ChunkWhen():
@@ -85,8 +133,10 @@ def test_ChunkWhen():
     assert [] >> ChunkWhen(func) >> Map(list) >> Collect() == []
     expected = ['|12', '|345', '|6']
     '|12|345|6' >> ChunkWhen(func) >> Map(''.join) >> Collect() == expected
+    '|12|345|6' >> ChunkWhen(func, ''.join) >> Collect() == expected
     expected = ['123456']
     assert '123456' >> ChunkWhen(func) >> Map(''.join) >> Collect() == expected
+    assert '123456' >> ChunkWhen(func, ''.join) >> Collect() == expected
 
 
 def test_ChunkBy():
@@ -94,11 +144,12 @@ def test_ChunkBy():
     assert [] >> ChunkBy(func) >> Map(list) >> Collect() == []
     expected = [[1, 1], [2], [3, 3, 3]]
     [1, 1, 2, 3, 3, 3] >> ChunkBy(func) >> Map(list) >> Collect() == expected
+    [1, 1, 2, 3, 3, 3] >> ChunkBy(func, list) >> Collect() == expected
     func = lambda x: x < 3
     expected = [[1, 1, 2], [3, 3, 3]]
-    [1, 1, 2, 3, 3, 3] >> ChunkBy(func) >> Map(list) >> Collect() == expected
+    [1, 1, 2, 3, 3, 3] >> ChunkBy(func, list) >> Collect() == expected
     expected = [[1, 2], [3], [1, 2], [3]]
-    [1, 2, 3, 1, 2, 3] >> ChunkBy(func) >> Map(list) >> Collect()
+    [1, 2, 3, 1, 2, 3] >> ChunkBy(func, list) >> Collect() == expected
 
 
 def test_Flatten():
@@ -117,6 +168,10 @@ def test_FlattenCol():
     assert data >> FlattenCol((1, 0)) >> Collect() == [(3, 1), (4, 2)]
     assert data >> FlattenCol((1, 1, 0)) >> Collect() == [(3, 3, 1), (4, 4, 2)]
 
+    data = [([1, 2], 3)]
+    assert data >> FlattenCol((0, 1)) >> Collect() == [(1, 3), (2, 3)]
+    assert data >> FlattenCol(0) >> Collect() == [(1,), (2,)]
+
 
 def test_FlatMap():
     assert [] >> FlatMap(_) >> Collect() == []
@@ -147,6 +202,18 @@ def test_FilterFalse():
     assert [0, 1, 2, 3] >> FilterFalse(_ < 0) >> Collect() == [0, 1, 2, 3]
 
 
+def test_FilterCol():
+    is_even = lambda n: n % 2 == 0
+    data = [(0, 'e'), (1, 'o'), (2, 'e')]
+    expected = [(0, 'e'), (2, 'e')]
+    assert data >> FilterCol(0, is_even) >> Collect() == expected
+
+    def same(s): return s[0] == s[1]
+    data = ['a_a', 'a_b', 'b_b']
+    expected = ['a_a', 'b_b']
+    assert data >> FilterCol((0, 2), same) >> Collect() == expected
+
+
 def test_Partition():
     smaller, larger = Range(5) >> Partition(_ < 3)
     assert smaller >> Collect() == [0, 1, 2]
@@ -233,8 +300,8 @@ def test_Pick():
         [1, 2, 3] >> Pick(-1) >> Consume()
     assert str(ex.value).startswith('p_n must not be negative')
 
-    assert Range(5) >> Pick(0.5, rnd.Random(0)) >> Collect() == [2, 3]
-    assert Range(5) >> Pick(0.7, rnd.Random(0)) >> Collect() == [2, 3, 4]
+    assert Range(5) >> Pick(0.5, StableRandom(1)) >> Collect() == [0, 4]
+    assert Range(5) >> Pick(0.7, StableRandom(0)) >> Collect() == [0, 1, 4]
 
     assert Range(10) >> Pick(1.0) >> Count() == 10
     assert Range(10) >> Pick(0.0) >> Count() == 0
@@ -269,7 +336,7 @@ def test_GroupBy():
 def test_GroupBySorted():
     @nut_sink
     def KV2List(iterable):
-        return iterable >> Map(lambda (k, es): (k, list(es))) >> Collect()
+        return iterable >> Map(lambda t: (t[0], list(t[1]))) >> Collect()
 
     @nut_sink
     def V2List(iterable):
@@ -301,7 +368,7 @@ def test_Clone():
 
 
 def test_Shuffle():
-    data = range(50)
+    data = list(range(50))
     assert data >> Shuffle(100) >> Collect() != data
     assert data >> Shuffle(100) >> Collect(set) == set(data)
 
@@ -310,8 +377,8 @@ def test_Shuffle():
 
     assert data >> Shuffle(1) >> Collect() == data
 
-    shuffled1 = data >> Shuffle(10, rand=rnd.Random(0)) >> Collect()
-    shuffled2 = data >> Shuffle(10, rand=rnd.Random(0)) >> Collect()
+    shuffled1 = data >> Shuffle(10, rand=StableRandom(0)) >> Collect()
+    shuffled2 = data >> Shuffle(10, rand=StableRandom(0)) >> Collect()
     assert shuffled1 == shuffled2
 
 
@@ -332,47 +399,87 @@ def test_MapMulti():
 
 def test_MapPar():
     assert [-1, -2, -3] >> MapPar(abs) >> Collect() == [1, 2, 3]
-    data = range(1000)
+    data = list(range(1000))
     assert data >> MapPar(_) >> MapPar(_) >> Collect() == data
 
 
 def test_Prefetch():
-    it = iter([1, 2, 3, 4])
+    data = [1, 2, 3, 4]
+    it = iter(data)
     assert it >> Prefetch() >> Take(1) >> Collect() == [1]
     assert next(it) == 3
 
-    it = iter([1, 2, 3, 4])
+    it = iter(data)
     assert it >> Prefetch() >> Take(2) >> Collect() == [1, 2]
     assert next(it) == 4
 
+    it = iter(data)
+    assert it >> Prefetch() >> Prefetch(2) >> Collect() == data
+
 
 def test_Cache():
     data = [(3, 'a'), (1, 'b'), (2, 'c')]
     with Cache() as cache:
         it = iter(data)
-        assert cache._dirpath is None
+        assert cache.path is None
         assert it >> cache >> Collect() == data
-        dirpath = cache._dirpath
-        assert os.path.isdir(dirpath)
+        cachepath = cache.path
+        assert os.path.isdir(cachepath)
         assert it >> cache >> Collect() == data
-    assert not os.path.isdir(dirpath)
-    assert cache._dirpath is None
+    assert not os.path.isdir(cachepath)
+    assert cache.path is None
 
-    with pytest.raises(ValueError) as ex:
-        [1] >> Cache(storage='memory') >> Consume()
-    assert str(ex.value).startswith('Unsupported storage')
-
-    data = range(100)
+    data = list(range(100))
     with Cache() as cache:
         it = iter(data)
         assert it >> cache >> Collect() == data
         assert it >> cache >> Collect() == data
+        assert cache >> Collect() == data
+
+    cache = Cache('tests/data/cache')
+    it = iter(data)
+    assert it >> cache >> Collect() == data
+    assert it >> cache >> Collect() == data
+    cache.clear()
+    assert not os.path.exists('tests/data/cache')
+
+    cache = Cache('tests/data/cache')
+    assert data >> cache >> Collect() == data
+    cache = Cache('tests/data/cache')
+    assert not os.path.exists('tests/data/cache')
 
 
 def test_PrintProgress():
     with Redirect() as out:
         numbers = range(3)
-        numbers >> PrintProgress(numbers, 0) >> Consume()
+        numbers >> PrintProgress(numbers, 'nums:', 0) >> Consume()
     expected = \
-        '\rprogress: 0% \rprogress: 50% \rprogress: 100% \rprogress: 100% \n'
+        '\rnums: 0% \rnums: 50% \rnums: 100% \rnums: 100% \n'
     assert out.getvalue() == expected
+
+
+def test_Try():
+    div = lambda x: 10 // x
+    Div = nut_function(div)
+
+    assert [] >> Try(div) >> Collect() == []
+    assert [10, 5, 1] >> Try(div) >> Collect() == [1, 2, 10]
+    assert [10, 0, 1] >> Try(Div(), -1) >> Collect() == [1, -1, 10]
+    assert [10, 0, 1] >> Try(Div(), 'IGNORE') >> Collect() == [1, 10]
+
+    ifzero = lambda x, e: str(x)
+    assert [10, 0, 1] >> Try(Div(), ifzero) >> Collect() == [1, '0', 10]
+
+    with Redirect('STDOUT') as out:
+        result = [10, 0, 1] >> Try(div, 'STDOUT') >> Collect()
+        assert result == [1, 10]
+    assert out.getvalue() == 'ERROR: 0 : integer division or modulo by zero\n'
+
+    with Redirect('STDERR') as out:
+        result = [10, 0, 1] >> Try(div) >> Collect()
+        assert result == [1, 10]
+    assert out.getvalue() == 'ERROR: 0 : integer division or modulo by zero\n'
+
+    with pytest.raises(TypeError) as ex:
+        [1, 0, 3] >> Try(Map(Div())) >> Collect()
+    assert str(ex.value).startswith("Need (nut) function")
